//EPWM FirmWare Source File
#include "can_fw.h"
#include "epwm_fw.h"
#include "config_fw.h"
#include "driverlib.h"
#include "device.h"
#include "SFO_V8.h"
#include "conv_fw.h"
#include "sw_timer.h"
#include "sdfm_fw.h"
#include "cmpss.h"


#define EPWM_MAX_DEADBAND_TIME       300U//4840U//4000U //This if for slow start, it will start with max DB then reduce little by little,
                                                         //So that the PWM will increase slowly.Avoid capacitor Current spikes.
#define EPWM_DEADBAND_STEP_SIZE      10U//10U//100U
#define EPWM_CLOCK_CYCLE             10U //period in nanoSeconds
#define EPWM_SHIFT_AMOUNT            7U
#define EPWM_DEADBAND_VALUE(X)       ((X / EPWM_CLOCK_CYCLE) << EPWM_SHIFT_AMOUNT)

#define EPWM_PRIM_PW_STEP     20
#define EPWM_SEC_PW_STEP      20
#define EPWM_PHASE_SHIFT_STEP  10

#define EPWM_SHIFT_STEP_SIZE  50U
#define EPWM_DEFAULT_PHASE_SHIFT 30U
#define EPWM_ZERO_PHASE_SHIFT  0U

#define EPWM_GPIO_GD_EN_PRI  9U
#define EPWM_GPIO_GD_EN_SEC  11U

#define PWM_COUNT          5
#define CHANNEL_A          1
#define CHANNEL_B          2
#define CHANNEL_AB         3

#define EPWM_TIMER_TBPRD   1000U
//Macros for PWM1
#define EPWM1_TIMER_TBPRD  1000U//to be changed to 5us
#define EPWM1_CMPA         500U
#define EPWM1_CMPB         500U
//Macros for PWM2
#define EPWM2_TIMER_TBPRD  1000U//to be changed to 5us
#define EPWM2_CMPA         500U
#define EPWM2_CMPB         500U

//Macros for PWM3
#define EPWM3_TIMER_TBPRD  1000U//300U
#define EPWM3_CMPA         500U//150U
#define EPWM3_CMPB         500U//150U
//Macros for PWM4
#define EPWM4_TIMER_TBPRD  1000U//300U
#define EPWM4_CMPA         500U//150U
#define EPWM4_CMPB         500U//150U
//Count Mode
#define EPWM_CMP_UP           1
#define EPWM_CMP_DOWN         0

typedef struct
{
    uint8_t  updateFlag;
    uint8_t  epwmPort;
    uint32_t epwmModule;
    uint16_t epwmPeriod;
    uint16_t epwmTimerIntCount;
    uint16_t epwmCompA;
    uint16_t epwmCompB;
    uint32_t phaseShift;
    uint16_t red;
    uint16_t fed;
}Epwm_Info_t; //now

typedef enum
{
    EPWM_ENABLE_PRIMARY = 0,
    EPWM_ENABLE_SECONDARY,
    EPWM_ENABLE_BOTH,
    EPWM_DISABLE_PRIMARY,
    EPWM_DISABLE_SECONDARY,
    EPWM_DISABLE_BOTH,
}epwm_switch_control_e;

typedef enum
{
    EPWM_IDLE,
    EPWM_STEADY_STATE,
    EPWM_TRIP_STATE,
}epwm_state_e;

typedef enum
{
    PWM_NO_EVENT = 0,
    PWM_START,
    PWM_STABLE,
    PWM_TRIP,
    PWM_STOP,
    PWM_UPDATE_VALUES,
}epwm_event_e;

typedef struct
{
  epwm_state_e state;
  uint16_t counter;	    //Counter was made to take care of slow start period to become stable
  uint16_t deadBand; 	//to keep track of DeadBand value
  uint16_t curr_prim_pulse_ns;	//to keep track of pulse width of Primary( Btw Pwm1 and Pwm2)
  uint16_t curr_sec_pulse_ns;  	//to keep track of pulse width of Secondary( Btw Pwm3 and Pwm4)
  int16_t curr_phase_pri_sec_ns; //to keep track of phase shift between epwm1 and epwm3
  uint16_t final_prim_pulse_ns;   //to track final pulse to be achieved
  uint16_t final_sec_pulse_ns;    //to track final pulse to be achieved
  int16_t final_phase_pri_sec_ns;//to track final phase to be achieved
  int16_t pwm1_time_base;	
  int16_t pwm2_time_base;
  int16_t pwm3_time_base;
  int16_t pwm4_time_base;
}epwm_fw_state_t;


Epwm_Info_t epwmInfo[PWM_COUNT];
epwm_fw_state_t epwm_fw_state = {EPWM_IDLE, 0, EPWM_MAX_DEADBAND_TIME, EPWM_ZERO_PHASE_SHIFT};

volatile uint32_t ePWM[] =
    {0, EPWM1_BASE, EPWM2_BASE, EPWM3_BASE, EPWM4_BASE};
int MEP_ScaleFactor; // Global variable used by the SFO library
                     // Result can be used for all HRPWM channels
                     // This variable is also copied to HRMSTEP
                     // register by SFO() function.



/******************************************************** #Init *********************************************************/
void EPWM_Init(void)
{
    uint16_t status = 0;

    epwm_ConfigureIOs();

    //
    // Calling SFO() updates the HRMSTEP register with calibrated MEP_ScaleFactor.
    // HRMSTEP must be populated with a scale factor value prior to enabling
    // high resolution period control.
    //
    while(status == SFO_INCOMPLETE)
    {
        status = SFO();
        if(status == SFO_ERROR)
        {
          ESTOP0;
        }              // steps/coarse step exceeds maximum of 255.
    }

    epwm_PreConfigureChannel(EPWM1_BASE);
    epwm_PreConfigureChannel(EPWM2_BASE);
    epwm_PreConfigureChannel(EPWM3_BASE);
    epwm_PreConfigureChannel(EPWM4_BASE);

    //
    // Interrupts that are used in this example are re-mapped to ISR functions
    // found within this file.
    //
    Interrupt_register(INT_EPWM1_TZ, &epwm1TZISR);
    Interrupt_register(INT_EPWM2_TZ, &epwm2TZISR);
    Interrupt_register(INT_EPWM3_TZ, &epwm3TZISR);
    Interrupt_register(INT_EPWM4_TZ, &epwm4TZISR);

    //
    // Assign the interrupt service routines to ePWM interrupts
    // Interrupts commented to avoid frequent interruption
    // earlier used for single pulse
    //
    Interrupt_register(INT_EPWM1, &epwm1_isr);
//    Interrupt_register(INT_EPWM2, &epwm2_isr);
//    Interrupt_register(INT_EPWM3, &epwm3_isr);
//    Interrupt_register(INT_EPWM4, &epwm4_isr);

    // Disable sync(Freeze clock to PWM as well). GTBCLKSYNC is applicable
    // only for multiple core devices. Uncomment the below statement if
    // applicable.
    //
    SysCtl_disablePeripheral(SYSCTL_PERIPH_CLK_TBCLKSYNC);

    epwm_TripZoneInit();

    //Initializing with Maximum Dead-band Time
    epwm_fw_state.deadBand = EPWM_MAX_DEADBAND_TIME;

    //
    // Initialize PWM1 without phase shift as master
    //
    epwm_ConfigureChannel(EPWM1_BASE);
    epwm_HRPWM_DeadBand(1, EPWM_DEADBAND_VALUE(epwm_fw_state.deadBand));
    //
    // Initialize PWM2 with phase shift of 300 TBCLKs
    //
    epwm_ConfigureChannel(EPWM2_BASE);
    EPWM_selectPeriodLoadEvent(EPWM2_BASE, EPWM_SHADOW_LOAD_MODE_SYNC);
    epwm_HRPWM_DeadBand(2, EPWM_DEADBAND_VALUE(epwm_fw_state.deadBand));

    //
    // Initialize PWM3 with phase shift of 600 TBCLKs
    //
    epwm_ConfigureChannel(EPWM3_BASE);
    EPWM_selectPeriodLoadEvent(EPWM3_BASE, EPWM_SHADOW_LOAD_MODE_SYNC);
    epwm_HRPWM_DeadBand(3, EPWM_DEADBAND_VALUE(epwm_fw_state.deadBand));

    //
    // Initialize PWM4 with phase shift of 900 TBCLKs
    //
    epwm_ConfigureChannel(EPWM4_BASE);
    EPWM_selectPeriodLoadEvent(EPWM4_BASE, EPWM_SHADOW_LOAD_MODE_SYNC);
    epwm_HRPWM_DeadBand(4, EPWM_DEADBAND_VALUE(epwm_fw_state.deadBand));

    //
    // ePWM1 SYNCO is generated on CTR=0
    //
    EPWM_setSyncOutPulseMode(EPWM1_BASE, EPWM_SYNC_OUT_PULSE_ON_COUNTER_ZERO);

    //
    // ePWM2 uses the ePWM 1 SYNCO as its SYNCIN.
    // ePWM2 SYNCO is generated from its SYNCIN, which is ePWM1 SYNCO
    //
    EPWM_setSyncOutPulseMode(EPWM2_BASE, EPWM_SYNC_OUT_PULSE_ON_EPWMxSYNCIN);

    //
    // ePWM4 uses the ePWM 1 SYNCO as its SYNCIN.
    //
    SysCtl_setSyncInputConfig(SYSCTL_SYNC_IN_EPWM4, SYSCTL_SYNC_IN_SRC_EPWM1SYNCOUT);

    //
    // Enable all phase shifts.
    //
    EPWM_enablePhaseShiftLoad(EPWM2_BASE);
    EPWM_enablePhaseShiftLoad(EPWM3_BASE);
    EPWM_enablePhaseShiftLoad(EPWM4_BASE);

    //to force pwms in tripped state at the time of initialization
    //uncomment below code to run pwms after reset
    EPWM_forceTripZoneEvent(EPWM1_BASE,EPWM_TZ_FORCE_EVENT_OST);
    EPWM_forceTripZoneEvent(EPWM2_BASE,EPWM_TZ_FORCE_EVENT_OST);
    EPWM_forceTripZoneEvent(EPWM3_BASE,EPWM_TZ_FORCE_EVENT_OST);
    EPWM_forceTripZoneEvent(EPWM4_BASE,EPWM_TZ_FORCE_EVENT_OST);

    //
    // Enable sync and clock to PWM
    //
    SysCtl_enablePeripheral(SYSCTL_PERIPH_CLK_TBCLKSYNC);


    epwm_Post_Init();

    //timer used for calling mep calibration function
    sw_timer_start(CONV_FW_MEP_CAL_TIMER_INDEX, CONV_FW_MED_CAL_TIMER_PERIOD_MS, SW_TIMER_PERIODIC);
    //timer used for updating the values into current status of pheripheral
    sw_timer_start(CONV_FW_PWM_UPDATE_TIMER_INDEX, CONV_FW_PWM_UPDATE_TIMER_PERIOD_MS, SW_TIMER_PERIODIC);
}


//
// Configure Output pins and Mux settings for 8 PWMs for gate driver control
//
static void epwm_ConfigureIOs(void)
{
    #ifdef _CONTROL_CARD
    //Primary side gate driver IN-/enable pin configuration as GPIO
    GPIO_setPadConfig(EPWM_GPIO_GD_EN_PRI, GPIO_PIN_TYPE_STD);
    GPIO_setDirectionMode(EPWM_GPIO_GD_EN_PRI, GPIO_DIR_MODE_OUT);
    epwm_Disable_Drivers_Primary();

    //Primary side gate driver IN-/enable pin configuration as GPIO
    GPIO_setPadConfig(EPWM_GPIO_GD_EN_SEC, GPIO_PIN_TYPE_STD);
    GPIO_setDirectionMode(EPWM_GPIO_GD_EN_SEC, GPIO_DIR_MODE_OUT);
    epwm_Disable_Drivers_Secondary();

#endif /* _CONTROL_CARD */


    //
    // EPWM1 -> EPWM1 Pinmux
    //
    GPIO_setPinConfig(GPIO_0_EPWM1A);
    GPIO_setPadConfig(1, GPIO_PIN_TYPE_STD);
    GPIO_setQualificationMode(1, GPIO_QUAL_SYNC);

    GPIO_setPinConfig(GPIO_1_EPWM1B);
    GPIO_setPadConfig(2, GPIO_PIN_TYPE_STD);
    GPIO_setQualificationMode(2, GPIO_QUAL_SYNC);

    //
    // EPWM2 -> EPWM2 Pinmux
    //
    GPIO_setPinConfig(GPIO_2_EPWM2A);
    GPIO_setPadConfig(3, GPIO_PIN_TYPE_STD);
    GPIO_setQualificationMode(3, GPIO_QUAL_SYNC);

    GPIO_setPinConfig(GPIO_3_EPWM2B);
    GPIO_setPadConfig(4, GPIO_PIN_TYPE_STD);
    GPIO_setQualificationMode(4, GPIO_QUAL_SYNC);

    //
    // EPWM3 -> EPWM3 Pinmux
    //
    GPIO_setPinConfig(GPIO_4_EPWM3A);
    GPIO_setPadConfig(5, GPIO_PIN_TYPE_STD);
    GPIO_setQualificationMode(5, GPIO_QUAL_SYNC);

    GPIO_setPinConfig(GPIO_5_EPWM3B);
    GPIO_setPadConfig(6, GPIO_PIN_TYPE_STD);
    GPIO_setQualificationMode(6, GPIO_QUAL_SYNC);

    //
    // EPWM4 -> EPWM4 Pinmux
    //
    GPIO_setPinConfig(GPIO_6_EPWM4A);
    GPIO_setPadConfig(7, GPIO_PIN_TYPE_STD);
    GPIO_setQualificationMode(7, GPIO_QUAL_SYNC);

    GPIO_setPinConfig(GPIO_7_EPWM4B);
    GPIO_setPadConfig(8, GPIO_PIN_TYPE_STD);
    GPIO_setQualificationMode(8, GPIO_QUAL_SYNC);
}


static void epwm_PreConfigureChannel(uint32_t base)
{
    EPWM_setClockPrescaler(base, EPWM_CLOCK_DIVIDER_1, EPWM_HSCLOCK_DIVIDER_1);
    EPWM_setTimeBasePeriod(base, 0);
    EPWM_setTimeBaseCounter(base, 0);
    EPWM_setTimeBaseCounterMode(base, EPWM_COUNTER_MODE_STOP_FREEZE);
    EPWM_disablePhaseShiftLoad(base);
    EPWM_setPhaseShift(base, 0);
    EPWM_setCounterCompareValue(base, EPWM_COUNTER_COMPARE_A, 0);
    EPWM_setCounterCompareShadowLoadMode(base, EPWM_COUNTER_COMPARE_A, EPWM_COMP_LOAD_ON_CNTR_ZERO);
    EPWM_setCounterCompareValue(base, EPWM_COUNTER_COMPARE_B, 0);
    EPWM_setCounterCompareShadowLoadMode(base, EPWM_COUNTER_COMPARE_B, EPWM_COMP_LOAD_ON_CNTR_ZERO);
    EPWM_setActionQualifierAction(base, EPWM_AQ_OUTPUT_A, EPWM_AQ_OUTPUT_NO_CHANGE, EPWM_AQ_OUTPUT_ON_TIMEBASE_ZERO);
    EPWM_setActionQualifierAction(base, EPWM_AQ_OUTPUT_A, EPWM_AQ_OUTPUT_NO_CHANGE, EPWM_AQ_OUTPUT_ON_TIMEBASE_PERIOD);
    EPWM_setActionQualifierAction(base, EPWM_AQ_OUTPUT_A, EPWM_AQ_OUTPUT_NO_CHANGE, EPWM_AQ_OUTPUT_ON_TIMEBASE_UP_CMPA);
    EPWM_setActionQualifierAction(base, EPWM_AQ_OUTPUT_A, EPWM_AQ_OUTPUT_NO_CHANGE, EPWM_AQ_OUTPUT_ON_TIMEBASE_DOWN_CMPA);
    EPWM_setActionQualifierAction(base, EPWM_AQ_OUTPUT_A, EPWM_AQ_OUTPUT_NO_CHANGE, EPWM_AQ_OUTPUT_ON_TIMEBASE_UP_CMPB);
    EPWM_setActionQualifierAction(base, EPWM_AQ_OUTPUT_A, EPWM_AQ_OUTPUT_NO_CHANGE, EPWM_AQ_OUTPUT_ON_TIMEBASE_DOWN_CMPB);
    EPWM_setActionQualifierAction(base, EPWM_AQ_OUTPUT_B, EPWM_AQ_OUTPUT_NO_CHANGE, EPWM_AQ_OUTPUT_ON_TIMEBASE_ZERO);
    EPWM_setActionQualifierAction(base, EPWM_AQ_OUTPUT_B, EPWM_AQ_OUTPUT_NO_CHANGE, EPWM_AQ_OUTPUT_ON_TIMEBASE_PERIOD);
    EPWM_setActionQualifierAction(base, EPWM_AQ_OUTPUT_B, EPWM_AQ_OUTPUT_NO_CHANGE, EPWM_AQ_OUTPUT_ON_TIMEBASE_UP_CMPA);
    EPWM_setActionQualifierAction(base, EPWM_AQ_OUTPUT_B, EPWM_AQ_OUTPUT_NO_CHANGE, EPWM_AQ_OUTPUT_ON_TIMEBASE_DOWN_CMPA);
    EPWM_setActionQualifierAction(base, EPWM_AQ_OUTPUT_B, EPWM_AQ_OUTPUT_NO_CHANGE, EPWM_AQ_OUTPUT_ON_TIMEBASE_UP_CMPB);
    EPWM_setActionQualifierAction(base, EPWM_AQ_OUTPUT_B, EPWM_AQ_OUTPUT_NO_CHANGE, EPWM_AQ_OUTPUT_ON_TIMEBASE_DOWN_CMPB);
}

static void epwm_TripZoneInit()
{
    //This is tripping for EPWM1 with Filter3 Using Mux20
    //
    //XBAR configuration for mapping SDFM threshold trigger event to Digital Compare submodule
    //
    //Trip Zone
    //
    //Muxing the trip4 signal to all SDFM Filter Comparators.
    //
    XBAR_setEPWMMuxConfig(XBAR_TRIP4, XBAR_EPWM_MUX16_SD1FLT1_COMPH_OR_COMPL);
    XBAR_enableEPWMMux(XBAR_TRIP4, XBAR_MUX16);

    XBAR_setEPWMMuxConfig(XBAR_TRIP4, XBAR_EPWM_MUX18_SD1FLT2_COMPH_OR_COMPL);
    XBAR_enableEPWMMux(XBAR_TRIP4, XBAR_MUX18);

    XBAR_setEPWMMuxConfig(XBAR_TRIP4, XBAR_EPWM_MUX20_SD1FLT3_COMPH_OR_COMPL);
    XBAR_enableEPWMMux(XBAR_TRIP4, XBAR_MUX20);

    XBAR_setEPWMMuxConfig(XBAR_TRIP4, XBAR_EPWM_MUX22_SD1FLT4_COMPH_OR_COMPL);
    XBAR_enableEPWMMux(XBAR_TRIP4, XBAR_MUX22);

    XBAR_setEPWMMuxConfig(XBAR_TRIP4, XBAR_EPWM_MUX00_CMPSS1_CTRIPH);
    XBAR_enableEPWMMux(XBAR_TRIP4, XBAR_MUX00);

    XBAR_setEPWMMuxConfig(XBAR_TRIP4, XBAR_EPWM_MUX02_CMPSS2_CTRIPH);
    XBAR_enableEPWMMux(XBAR_TRIP4, XBAR_MUX02);

    //////////////////////////------ EPWM1 With TRIPIN4-----//////////////////////////////////////////
    //
    // Select TRIPIN1 (INPUT XBAR INPUT1) as the source for DCAH
    //
    EPWM_selectDigitalCompareTripInput(
            EPWM1_BASE, EPWM_DC_TRIP_TRIPIN4, EPWM_DC_TYPE_DCAH);

    //
    // DCAEVT1 is generated when DCAH is low
    //
    EPWM_setTripZoneDigitalCompareEventCondition(
            EPWM1_BASE, EPWM_TZ_DC_OUTPUT_A1, EPWM_TZ_EVENT_DCXH_HIGH);

    //
    // DCAEVT1 uses the unfiltered version of DCAEVT1
    //
    EPWM_setDigitalCompareEventSource(
            EPWM1_BASE, EPWM_DC_MODULE_A, EPWM_DC_EVENT_1, EPWM_DC_EVENT_SOURCE_ORIG_SIGNAL);

    //
    // DCAEVT1 is asynchronous
    //
    EPWM_setDigitalCompareEventSyncMode(
            EPWM1_BASE, EPWM_DC_MODULE_A, EPWM_DC_EVENT_1, EPWM_DC_EVENT_INPUT_NOT_SYNCED);

    //
    // Action on DCAEVT1
    // Force the EPWMA LOW
    //
    EPWM_setTripZoneAction(EPWM1_BASE,
                           EPWM_TZ_ACTION_EVENT_TZA,
                           EPWM_TZ_ACTION_LOW);
    EPWM_setTripZoneAction(EPWM1_BASE,
                           EPWM_TZ_ACTION_EVENT_TZB,
                           EPWM_TZ_ACTION_LOW);
    EPWM_enableTripZoneInterrupt(EPWM1_BASE, EPWM_TZ_INTERRUPT_DCAEVT1);


    //////////////////////////------ EPWM2 With TRIPIN4-----//////////////////////////////////////////
    //
    // Select TRIPIN1 (INPUT XBAR INPUT1) as the source for DCAH
    //
    EPWM_selectDigitalCompareTripInput(
            EPWM2_BASE, EPWM_DC_TRIP_TRIPIN4, EPWM_DC_TYPE_DCAH);

    //
    // DCAEVT1 is generated when DCAH is low
    //
    EPWM_setTripZoneDigitalCompareEventCondition(
            EPWM2_BASE, EPWM_TZ_DC_OUTPUT_A1, EPWM_TZ_EVENT_DCXH_HIGH);

    //
    // DCAEVT1 uses the unfiltered version of DCAEVT1
    //
    EPWM_setDigitalCompareEventSource(
            EPWM2_BASE, EPWM_DC_MODULE_A, EPWM_DC_EVENT_1, EPWM_DC_EVENT_SOURCE_ORIG_SIGNAL);

    //
    // DCAEVT1 is asynchronous
    //
    EPWM_setDigitalCompareEventSyncMode(
            EPWM2_BASE, EPWM_DC_MODULE_A, EPWM_DC_EVENT_1, EPWM_DC_EVENT_INPUT_NOT_SYNCED);

    //
    // Action on DCAEVT1
    // Force the EPWMA LOW
    //
    EPWM_setTripZoneAction(EPWM2_BASE,
                           EPWM_TZ_ACTION_EVENT_TZA,
                           EPWM_TZ_ACTION_LOW);
    EPWM_setTripZoneAction(EPWM2_BASE,
                           EPWM_TZ_ACTION_EVENT_TZB,
                           EPWM_TZ_ACTION_LOW);
    EPWM_enableTripZoneInterrupt(EPWM2_BASE, EPWM_TZ_INTERRUPT_DCAEVT1);


    //////////////////////////------ EPWM3 With TRIPIN4-----//////////////////////////////////////////

    EPWM_selectDigitalCompareTripInput(
            EPWM3_BASE, EPWM_DC_TRIP_TRIPIN4, EPWM_DC_TYPE_DCAH);

    //
    // DCAEVT1 is generated when DCAH is low
    //
    EPWM_setTripZoneDigitalCompareEventCondition(
            EPWM3_BASE, EPWM_TZ_DC_OUTPUT_A1, EPWM_TZ_EVENT_DCXH_HIGH);

    //
    // DCAEVT1 uses the unfiltered version of DCAEVT1
    //
    EPWM_setDigitalCompareEventSource(
            EPWM3_BASE, EPWM_DC_MODULE_A, EPWM_DC_EVENT_1, EPWM_DC_EVENT_SOURCE_ORIG_SIGNAL);

    //
    // DCAEVT1 is asynchronous
    //
    EPWM_setDigitalCompareEventSyncMode(
            EPWM3_BASE, EPWM_DC_MODULE_A, EPWM_DC_EVENT_1, EPWM_DC_EVENT_INPUT_NOT_SYNCED);

    //
    // Action on DCAEVT1
    // Force the EPWMA LOW
    //
    EPWM_setTripZoneAction(EPWM3_BASE,
                           EPWM_TZ_ACTION_EVENT_TZA,
                           EPWM_TZ_ACTION_LOW);
    EPWM_setTripZoneAction(EPWM3_BASE,
                           EPWM_TZ_ACTION_EVENT_TZB,
                           EPWM_TZ_ACTION_LOW);
    EPWM_enableTripZoneInterrupt(EPWM3_BASE, EPWM_TZ_INTERRUPT_DCAEVT1);



    //////////////////////////------ EPWM4 With TRIPIN4-----//////////////////////////////////////////

    EPWM_selectDigitalCompareTripInput(
            EPWM4_BASE, EPWM_DC_TRIP_TRIPIN4, EPWM_DC_TYPE_DCAH);

    //
    // DCAEVT1 is generated when DCAH is low
    //
    EPWM_setTripZoneDigitalCompareEventCondition(
            EPWM4_BASE, EPWM_TZ_DC_OUTPUT_A1, EPWM_TZ_EVENT_DCXH_HIGH);

    //
    // DCAEVT1 uses the unfiltered version of DCAEVT1
    //
    EPWM_setDigitalCompareEventSource(
            EPWM4_BASE, EPWM_DC_MODULE_A, EPWM_DC_EVENT_1, EPWM_DC_EVENT_SOURCE_ORIG_SIGNAL);

    //
    // DCAEVT1 is asynchronous
    //
    EPWM_setDigitalCompareEventSyncMode(
            EPWM4_BASE, EPWM_DC_MODULE_A, EPWM_DC_EVENT_1, EPWM_DC_EVENT_INPUT_NOT_SYNCED);

    //
    // Action on DCAEVT1
    // Force the EPWMA LOW
    //
    EPWM_setTripZoneAction(EPWM4_BASE,
                           EPWM_TZ_ACTION_EVENT_TZA,
                           EPWM_TZ_ACTION_LOW);
    EPWM_setTripZoneAction(EPWM4_BASE,
                           EPWM_TZ_ACTION_EVENT_TZB,
                           EPWM_TZ_ACTION_LOW);
    EPWM_enableTripZoneInterrupt(EPWM4_BASE, EPWM_TZ_INTERRUPT_DCAEVT1);



    Interrupt_enable(INT_EPWM1_TZ);
    Interrupt_enable(INT_EPWM2_TZ);
    Interrupt_enable(INT_EPWM3_TZ);
    Interrupt_enable(INT_EPWM4_TZ);

}


//
//
//
static void epwm_ConfigureChannel(uint32_t base)
{
    //
    // Set-up TBCLK
    //
    EPWM_setTimeBasePeriod(base, EPWM_TIMER_TBPRD-1);
    EPWM_setPhaseShift(base, 0U);
    EPWM_setTimeBaseCounter(base, 0U);

    //
    // Set Compare values
    //
    EPWM_setCounterCompareValue(base,
                                EPWM_COUNTER_COMPARE_A,
                                (EPWM_TIMER_TBPRD/2)-1);
    EPWM_setCounterCompareValue(base,
                                EPWM_COUNTER_COMPARE_B,
                                (EPWM_TIMER_TBPRD/2)-1);


    //
    // Set up counter mode
    //
    EPWM_setTimeBaseCounterMode(base, EPWM_COUNTER_MODE_UP);
    EPWM_disablePhaseShiftLoad(base);
    EPWM_setClockPrescaler(base,
                           EPWM_CLOCK_DIVIDER_1,
                           EPWM_HSCLOCK_DIVIDER_1);

    //
    // Set up shadowing
    //
    EPWM_setCounterCompareShadowLoadMode(base,
                                         EPWM_COUNTER_COMPARE_A,
                                         EPWM_COMP_LOAD_ON_CNTR_ZERO);
    EPWM_setCounterCompareShadowLoadMode(base,
                                         EPWM_COUNTER_COMPARE_B,
                                         EPWM_COMP_LOAD_ON_CNTR_ZERO);


    EPWM_setDeadBandControlShadowLoadMode(base, EPWM_DB_LOAD_ON_CNTR_ZERO);

    //
    // Set actions
    //
    EPWM_setActionQualifierAction(base,
                                  EPWM_AQ_OUTPUT_A,
                                  EPWM_AQ_OUTPUT_HIGH,
                                  EPWM_AQ_OUTPUT_ON_TIMEBASE_ZERO);
    EPWM_setActionQualifierAction(base,
                                  EPWM_AQ_OUTPUT_B,
                                  EPWM_AQ_OUTPUT_HIGH,
                                  EPWM_AQ_OUTPUT_ON_TIMEBASE_ZERO);
    EPWM_setActionQualifierAction(base,
                                  EPWM_AQ_OUTPUT_A,
                                  EPWM_AQ_OUTPUT_LOW,
                                  EPWM_AQ_OUTPUT_ON_TIMEBASE_UP_CMPA);
    EPWM_setActionQualifierAction(base,
                                  EPWM_AQ_OUTPUT_B,
                                  EPWM_AQ_OUTPUT_LOW,
                                  EPWM_AQ_OUTPUT_ON_TIMEBASE_UP_CMPB);



    HRPWM_setMEPEdgeSelect(base, HRPWM_CHANNEL_A, HRPWM_MEP_CTRL_RISING_AND_FALLING_EDGE);
    HRPWM_setMEPControlMode(base, HRPWM_CHANNEL_A, HRPWM_MEP_DUTY_PERIOD_CTRL);
    HRPWM_setCounterCompareShadowLoadEvent(base, HRPWM_CHANNEL_A, HRPWM_LOAD_ON_CNTR_ZERO_PERIOD);

    HRPWM_setMEPEdgeSelect(base, HRPWM_CHANNEL_B, HRPWM_MEP_CTRL_RISING_AND_FALLING_EDGE);
    HRPWM_setMEPControlMode(base, HRPWM_CHANNEL_B, HRPWM_MEP_DUTY_PERIOD_CTRL);
    HRPWM_setCounterCompareShadowLoadEvent(base, HRPWM_CHANNEL_B, HRPWM_LOAD_ON_CNTR_ZERO_PERIOD);

    HRPWM_enableAutoConversion(base);

    //
    // Turn on high-resolution period control.
    //

    HRPWM_enablePeriodControl(base);
    HRPWM_enablePhaseShiftLoad(base);

    EPWM_forceSyncPulse(base);


    //
    // Interrupt where we will change the Compare Values
    // Select INT on Time base counter zero event,
    // Enable INT, generate INT on 1rd event
    // below code not being used as epwm interrupts are disabled
    //
    EPWM_setInterruptSource(base, EPWM_INT_TBCTR_ZERO);
    EPWM_enableInterrupt(base);
    EPWM_setInterruptEventCount(base, 1U);
}

//
//
//
void epwm_Post_Init(void)
{
    EPWM_enableTripZoneSignals(EPWM1_BASE, EPWM_TZ_SIGNAL_DCAEVT1);
    EPWM_enableTripZoneSignals(EPWM2_BASE, EPWM_TZ_SIGNAL_DCAEVT1);
    EPWM_enableTripZoneSignals(EPWM3_BASE, EPWM_TZ_SIGNAL_DCAEVT1);
    EPWM_enableTripZoneSignals(EPWM4_BASE, EPWM_TZ_SIGNAL_DCAEVT1);
}



/******************************************************** #DeadBand *********************************************************/
//
// @fn: this will set DeadBand value with Active High RED on Both channels
// @param: base contains base value for DB to set. value contains actual value to set in (DBRED:DBREDHR)
// \b Note: redCount is a 21 bit value.
// \b Note: For configuring DBRED = 0x4, DBREDHR = 0x1; value of
//          redCount = ((0x4 << 7) | 0x1) = 0x201
//
static void epwm_HRPWM_DeadBand(uint8_t base, uint32_t value)
{
    //
    // DO NOT Switch Output A with Output B
    // true - it will swap the signal , ie. it will replicate other channel's signal
    // false -  it will produce same signal as it generates
    //@must - during complementary mode, these lines will set the swap mode, if we do these steps after setting up
    //        then our output might have some problems. so do this at beginning of init.
    EPWM_setDeadBandOutputSwapMode(ePWM[base], EPWM_DB_OUTPUT_A, false);
    EPWM_setDeadBandOutputSwapMode(ePWM[base], EPWM_DB_OUTPUT_B, false);
    //
    // Use EPWMA as the input for both RED and FED
    //
    EPWM_setRisingEdgeDeadBandDelayInput(ePWM[base], EPWM_DB_INPUT_EPWMA);
    EPWM_setFallingEdgeDeadBandDelayInput(ePWM[base], EPWM_DB_INPUT_EPWMB);

    //
    // Set the RED and FED values
    //
    HRPWM_setRisingEdgeDelay(ePWM[base], value);
    HRPWM_setFallingEdgeDelay(ePWM[base], value);

    //
    // Invert only the Falling Edge delayed output (AHC)
    //EPWM_DB_POLARITY_ACTIVE_HIGH - It will produce same signal as mentioned DB
    //EPWM_DB_POLARITY_ACTIVE_LOW - It will produce complement of the signal
    EPWM_setDeadBandDelayPolarity(ePWM[base], EPWM_DB_RED, EPWM_DB_POLARITY_ACTIVE_HIGH);
    EPWM_setDeadBandDelayPolarity(ePWM[base], EPWM_DB_FED, EPWM_DB_POLARITY_ACTIVE_LOW);

    //
    // Use the delayed signals instead of the original signals
    //
    EPWM_setDeadBandDelayMode(ePWM[base], EPWM_DB_RED, true);
    EPWM_setDeadBandDelayMode(ePWM[base], EPWM_DB_FED, true);


    HRPWM_setDeadbandMEPEdgeSelect(ePWM[base], HRPWM_DB_MEP_CTRL_RED_FED);


    HRPWM_setRisingEdgeDelayLoadMode(ePWM[base], HRPWM_LOAD_ON_CNTR_ZERO);


    HRPWM_setFallingEdgeDelayLoadMode(ePWM[base], HRPWM_LOAD_ON_CNTR_ZERO);

    TRACE_I(EPWM_TRACE,"EPWM%d Setted FED & RED with value = %d ...\r\n",(uint32_t)base,value);
}


/******************************************************** #Phase Shift *********************************************************/

void Epwm_Fw_SetPhaseShifts(uint16_t priShift, int16_t pri_sec_shift, uint16_t secShift)
{
    int16_t phase12 = priShift - epwm_fw_state.pwm1_time_base;
    int16_t phase13 = pri_sec_shift + phase12/2 - secShift/2;
    int16_t phase14 = pri_sec_shift + phase12/2 + secShift/2;
    if(phase12 <= 5000 && phase13 <= 5000 && phase14 <= 7500 && phase14 >= phase13)
    {
        epwm_fw_state.curr_prim_pulse_ns = priShift;
        epwm_fw_state.curr_sec_pulse_ns = secShift;
        epwm_fw_state.curr_phase_pri_sec_ns = pri_sec_shift;
        Epwm_Fw_SetPhase12(phase12);
        Epwm_Fw_SetPhase13(phase13);
        Epwm_Fw_SetPhase14(phase14);
    }
}
//It will alter Pulse width of Primary,
//sets phase shift on EPWM2 alone (Shift Btw Pwm1 and Pwm2)
void Epwm_Fw_SetPulseWidthPri(int16_t shiftValue)
{
    int16_t pulse = shiftValue - epwm_fw_state.pwm1_time_base;
    int16_t phase13 = epwm_fw_state.curr_phase_pri_sec_ns + pulse/2 - epwm_fw_state.curr_sec_pulse_ns/2;
    int16_t phase14 = epwm_fw_state.curr_phase_pri_sec_ns + pulse/2 + epwm_fw_state.curr_sec_pulse_ns/2;
    if(pulse <= 5000 && phase13 <= 5000 && phase14 <= 7500)
    {
        epwm_fw_state.curr_prim_pulse_ns = pulse;
        Epwm_Fw_SetPhase12(shiftValue);
        Epwm_Fw_SetPhase13(phase13);
        Epwm_Fw_SetPhase14(phase14);
    }
}

//It will alter Pulse width of Secondary,
//sets phase shift on Pmw3 and Pwm4 alone (Shift Btw Pwm3 and Pwm4)
void Epwm_Fw_SetPulseWidthSec(int16_t shiftValue)
{
    int16_t phase13 = epwm_fw_state.curr_phase_pri_sec_ns + epwm_fw_state.curr_prim_pulse_ns/2 - shiftValue/2;
    int16_t phase14 = epwm_fw_state.curr_phase_pri_sec_ns + epwm_fw_state.curr_prim_pulse_ns/2 + shiftValue/2;
    if(phase13 <= 5000 && phase14 <= 7500)
    {
        epwm_fw_state.curr_sec_pulse_ns = phase14 - phase13;
        Epwm_Fw_SetPhase13(phase13);
        Epwm_Fw_SetPhase14(phase14);
    }
}

//It will set the Phase shift between Primary to Secondary
void Epwm_Fw_SetPhaseShift_Pri_Sec(int16_t value)
{
    int16_t phase13 = value + epwm_fw_state.curr_prim_pulse_ns/2 - epwm_fw_state.curr_sec_pulse_ns/2;
    int16_t phase14 = value + epwm_fw_state.curr_prim_pulse_ns/2 + epwm_fw_state.curr_sec_pulse_ns/2;
    if(phase13 <= 5000 && phase14 <= 7500 && phase14 >= phase13)
    {
        epwm_fw_state.curr_phase_pri_sec_ns = value;
        Epwm_Fw_SetPhase13(phase13);
        Epwm_Fw_SetPhase14(phase14);
    }
}

//It will set Phase shift on PWM2 w.r.t PWM1
void Epwm_Fw_SetPhase12(int16_t value)
{
//    epwm_fw_state.pwm2_time_base = value;
    uint32_t shift2Count = (EPWM_TIMER_TBPRD - (value/10))%EPWM_TIMER_TBPRD;
    uint8_t shift2CountHr = 0;

    shift2Count <<= 8;
    shift2Count += shift2CountHr;

//    HRPWM_setPhaseShift(EPWM2_BASE, shift2Count);
    epwmInfo[2].phaseShift = shift2Count;
    epwmInfo[2].updateFlag = TRUE;
}

//It will set Phase shift on PWM3 w.r.t PWM1
void Epwm_Fw_SetPhase13(int16_t value)
{
    epwm_fw_state.pwm3_time_base = value;
    uint32_t shift3Count = (EPWM_TIMER_TBPRD - (value/10))%EPWM_TIMER_TBPRD;
    uint8_t shift3CountHr = 0;

    shift3Count <<= 8;
    shift3Count += shift3CountHr;

//    HRPWM_setPhaseShift(EPWM3_BASE, shift3Count);
    epwmInfo[3].phaseShift = shift3Count;
    epwmInfo[3].updateFlag = TRUE;
}

//It will set Phase shift on PWM4 w.r.t PWM1
void Epwm_Fw_SetPhase14(int16_t value)
{
    epwm_fw_state.pwm4_time_base = value;
    uint32_t shift4Count = (EPWM_TIMER_TBPRD - (value/10))%EPWM_TIMER_TBPRD;
    uint8_t shift4CountHr = 0;

    shift4Count <<= 8;
    shift4Count += shift4CountHr;

//    HRPWM_setPhaseShift(EPWM4_BASE, shift4Count);
    epwmInfo[4].phaseShift = shift4Count;
    epwmInfo[4].updateFlag = TRUE;
}

//todo - implement using soft timer
void Epwm_Fw_mepCalibration(void)
{
    uint16_t status = SFO(); // in background, MEP calibration module
                             // continuously updates MEP_ScaleFactor

    if (sw_timer_process(CONV_FW_MEP_CAL_TIMER_INDEX) == 1)
        {
            if (status == 2)
            {
                ESTOP0;   // SFO function returns 2 if an error occurs & #
                // of MEP steps/coarse step
            }
        }
}


void Conv_Fw_UpdateValues()
{
  Epwm_Fw_ReadValues(epwm_values);

   if((epwm_values[2].updateFlag && epwm_values[3].updateFlag && epwm_values[4].updateFlag) == TRUE)
   {
      HWREG(EPWM2_BASE + HRPWM_O_TBPHS) = epwm_values[2].phaseShift << 8U;
      HWREG(EPWM3_BASE + HRPWM_O_TBPHS) = epwm_values[3].phaseShift << 8U;
      HWREG(EPWM4_BASE + HRPWM_O_TBPHS) = epwm_values[4].phaseShift << 8U;
   }

}


//
// epwm1TZISR - ePWM1 TZ ISR
//
__interrupt void epwm1TZISR(void)
{
    //
    // re-enable the interrupts
    //
    EPWM_clearTripZoneFlag(EPWM1_BASE, EPWM_TZ_INTERRUPT | EPWM_TZ_FLAG_DCAEVT1);

    //
    // Uncomment the below lines of code to re-enable
    // the interrupts and the ePWM output if using one-shot
    // Below interrupt to be handled by application
    // todo - implement callback to application
    // EPWM_clearTripZoneFlag(EPWM1_BASE, EPWM_TZ_FLAG_OST | EPWM_TZ_INTERRUPT | EPWM_TZ_FLAG_DCAEVT1);
    //

    // Acknowledge this interrupt to receive more interrupts from group 2.3
//    Interrupt_clearACKGroup(INTERRUPT_ACK_GROUP2);
}

//
// epwm2TZISR - ePWM2 TZ ISR
//
__interrupt void epwm2TZISR(void)
{
    //
//    // Uncomment the below lines of code to re-enable the interrupts
//    //
    EPWM_clearTripZoneFlag(EPWM2_BASE, EPWM_TZ_INTERRUPT | EPWM_TZ_FLAG_DCAEVT1);
//
//    //
//    // Uncomment the below lines of code to re-enable
//    // the interrupts and the ePWM output if using one-shot
//    //
//    EPWM_clearTripZoneFlag(EPWM2_BASE, EPWM_TZ_FLAG_OST | EPWM_TZ_INTERRUPT | EPWM_TZ_FLAG_DCAEVT1);
//
//    //
//    // Acknowledge this interrupt to receive more interrupts from group 2.3
//    Interrupt_clearACKGroup(INTERRUPT_ACK_GROUP2);
}

//
// epwm3TZISR - ePWM1 TZ ISR
//
__interrupt void epwm3TZISR(void)
{
    //
//    // Uncomment the below lines of code to re-enable the interrupts
//    //
    EPWM_clearTripZoneFlag(EPWM3_BASE, EPWM_TZ_INTERRUPT | EPWM_TZ_FLAG_DCAEVT1);
//
//    //
//    // Uncomment the below lines of code to re-enable
//    // the interrupts and the ePWM output if using one-shot
//    //
//    EPWM_clearTripZoneFlag(EPWM3_BASE, EPWM_TZ_FLAG_OST | EPWM_TZ_INTERRUPT | EPWM_TZ_FLAG_DCAEVT1);
//
//    //
//    // Acknowledge this interrupt to receive more interrupts from group 2.3
//    Interrupt_clearACKGroup(INTERRUPT_ACK_GROUP2);
}

//
// epwm4TZISR - ePWM1 TZ ISR
//
__interrupt void epwm4TZISR(void)
{
    //
//    // Uncomment the below lines of code to re-enable the interrupts
//    //
    EPWM_clearTripZoneFlag(EPWM4_BASE, EPWM_TZ_INTERRUPT | EPWM_TZ_FLAG_DCAEVT1);
//
//    //
//    // Uncomment the below lines of code to re-enable
//    // the interrupts and the ePWM output if using one-shot
//    //
//    EPWM_clearTripZoneFlag(EPWM4_BASE, EPWM_TZ_FLAG_OST | EPWM_TZ_INTERRUPT | EPWM_TZ_FLAG_DCAEVT1);
//
//    //
//    // Acknowledge this interrupt to receive more interrupts from group 2.3
//    Interrupt_clearACKGroup(INTERRUPT_ACK_GROUP2);
}




//
// epwm1_isr - ePWM 1 _isr
//
__interrupt void epwm1_isr(void) //ISR generated for every time period (10uS)
{
    //
    // Clear INT flag for this timer
    //
    EPWM_clearEventTriggerInterruptFlag(EPWM1_BASE);

    //
    // Acknowledge interrupt group
    //
    Interrupt_clearACKGroup(INTERRUPT_ACK_GROUP3);
}

//
// epwm2_isr - ePWM 2 _isr
//
__interrupt void epwm2_isr(void)
{
    //
    // Clear INT flag for this timer
    //
    EPWM_clearEventTriggerInterruptFlag(EPWM2_BASE);

    //
    // Acknowledge interrupt group
    //
    Interrupt_clearACKGroup(INTERRUPT_ACK_GROUP3);
}

//
// epwm3_isr - ePWM 3 _isr
//
__interrupt void epwm3_isr(void)
{
    //
    // Clear INT flag for this timer
    //
    EPWM_clearEventTriggerInterruptFlag(EPWM3_BASE);

    //
    // Acknowledge interrupt group
    //
    Interrupt_clearACKGroup(INTERRUPT_ACK_GROUP3);
}

//
// epwm4_isr - ePWM 4 _isr
//
__interrupt void epwm4_isr(void)
{
    //
    // Clear INT flag for this timer
    //
    EPWM_clearEventTriggerInterruptFlag(EPWM4_BASE);

    //
    // Acknowledge interrupt group
    //
    Interrupt_clearACKGroup(INTERRUPT_ACK_GROUP3);
}

void Epwm_Fw_StepIncPriPulse(void)
{
    volatile int16_t pulse = 20 + epwm_fw_state.final_prim_pulse_ns;
    if(pulse <= 5000)
    {
        Epwm_Fw_SetPriPulseFinal(pulse);
//        Epwm_Fw_SetPulseWidthPri(pulse);
    }
}
void Epwm_Fw_StepDecPriPulse(void)
{
    int16_t pulse = epwm_fw_state.final_prim_pulse_ns;
    if(pulse >= 0)
    {
        Epwm_Fw_SetPriPulseFinal(pulse);
//        Epwm_Fw_SetPulseWidthPri(pulse);
    }
}

void Epwm_Fw_StepIncSecPulse(void)
{
    int16_t pulse = 20 + epwm_fw_state.final_sec_pulse_ns;
    if(pulse <= 5000)
    {
        Epwm_Fw_SetSecPulseFinal(pulse);
//        Epwm_Fw_SetPulseWidthSec(pulse);
    }
}
void Epwm_Fw_StepDecSecPulse(void)
{
    int16_t pulse = epwm_fw_state.final_sec_pulse_ns;
    if(pulse >= 0)
    {
        Epwm_Fw_SetSecPulseFinal(pulse);
//        Epwm_Fw_SetPulseWidthSec(pulse);
    }
}

//While changing phase shift, first call secondary pulse width function, then change phase.
void Epwm_Fw_StepIncPhase(void)
{
    int16_t phase = epwm_fw_state.final_phase_pri_sec_ns + 10;
    if(phase <= 2500)
    {
        Epwm_Fw_Set_Pri_Sec_PhaseFinal(phase);
//        Epwm_Fw_SetPhaseShift_Pri_Sec(phase);
    }
}
void Epwm_Fw_StepDecPhase(void)
{
    int16_t phase = epwm_fw_state.final_phase_pri_sec_ns - 10;
    if(phase >= 0)
    {
        Epwm_Fw_Set_Pri_Sec_PhaseFinal(phase);
//        Epwm_Fw_SetPhaseShift_Pri_Sec(phase);
    }
}

void Epwm_Fw_SetDeadBandAll(uint16_t value)
{
    uint8_t base = 0;
    for(base = 1; base <= PWM_COUNT; base++)
    {
        Epwm_Fw_SetRED(base, value);
        Epwm_Fw_SetFED(base, value);
    }

}

void Epwm_Fw_StepIncAll(void)
{
    int16_t value12 = epwm_fw_state.final_prim_pulse_ns + EPWM_PRIM_PW_STEP;
    int16_t value14 = epwm_fw_state.final_sec_pulse_ns + EPWM_SEC_PW_STEP;
    int16_t value13 = epwm_fw_state.final_phase_pri_sec_ns + EPWM_PHASE_SHIFT_STEP;

    if((value12 <= 5000) && (value13 <= 2500 && value13 >= EPWM_DEFAULT_PHASE_SHIFT) && (value14 <= 5000))
    {
        Epwm_Fw_Set_SetPhaseShiftsFinal(value12,value13,value14); //will set all values in final value struct
    }

}

void Epwm_Fw_StepDecAll(void)
{
    int16_t value12 = epwm_fw_state.final_prim_pulse_ns - EPWM_PRIM_PW_STEP;
    int16_t value14 = epwm_fw_state.final_sec_pulse_ns - EPWM_SEC_PW_STEP;
    int16_t value13 = epwm_fw_state.final_phase_pri_sec_ns - EPWM_PHASE_SHIFT_STEP;

    if((value12 <= 5000) && (value13 <= 2500 && value13 >= EPWM_DEFAULT_PHASE_SHIFT) && (value14 <= 5000))
    {
        Epwm_Fw_Set_SetPhaseShiftsFinal(value12,value13,value14); //will set all values in final value struct
    }

}
void Epwm_Fw_StepIncDeadBand(void)
{
    int16_t value = epwm_fw_state.deadBand;
    value += 10;
    if(value <= 500)
    {
        Epwm_Fw_SetDeadBandAll(value);
    }
}
void Epwm_Fw_StepDecDeadBand(void)
{
    int16_t value = epwm_fw_state.deadBand;
    value -= 10;
    if(value >= 150)
    {
        Epwm_Fw_SetDeadBandAll(value);
    }
}

void Epwm_Fw_Read(uint8_t* buff)
{
    buff[2] = epwm_fw_state.curr_prim_pulse_ns >> 8;
    buff[3] = epwm_fw_state.curr_prim_pulse_ns & 0x00FF;
    buff[4] = epwm_fw_state.curr_phase_pri_sec_ns >> 8;
    buff[5] = epwm_fw_state.curr_phase_pri_sec_ns & 0x00FF;
    buff[6] = epwm_fw_state.curr_sec_pulse_ns >> 8;
    buff[7] = epwm_fw_state.curr_sec_pulse_ns & 0x00FF;
}

int16_t Epwm_Fw_GetPulseWidthPri()
{
    return epwm_fw_state.curr_prim_pulse_ns;
}
int16_t Epwm_Fw_GetPulseWidthSec()
{
    return epwm_fw_state.curr_sec_pulse_ns;
}
int16_t Epwm_Fw_GetPhaseShift()
{
    return epwm_fw_state.curr_phase_pri_sec_ns;
}


void Epwm_Fw_ReadValues(Epwm_Values_t epwm_values[])
{
    epwm_values[2].phaseShift = epwmInfo[2].phaseShift;
    epwm_values[2].updateFlag = epwmInfo[2].updateFlag;
    epwmInfo[2].updateFlag = FALSE;
    epwm_values[3].phaseShift = epwmInfo[3].phaseShift;
    epwm_values[3].updateFlag = epwmInfo[3].updateFlag;
    epwmInfo[3].updateFlag = FALSE;
    epwm_values[4].phaseShift = epwmInfo[4].phaseShift;
    epwm_values[4].updateFlag = epwmInfo[4].updateFlag;
    epwmInfo[4].updateFlag = FALSE;
}

void Epwm_Fw_SetPhaseAll(uint16_t pri_pw, uint16_t sec_pw, uint16_t pri_sec_phase)
{
    epwm_fw_state.final_prim_pulse_ns = pri_pw;
    epwm_fw_state.final_sec_pulse_ns = sec_pw;
    epwm_fw_state.final_phase_pri_sec_ns = pri_sec_phase;
}

void Epwm_Fw_SetPriPulseFinal(uint16_t pri_pw)
{
    epwm_fw_state.final_prim_pulse_ns = pri_pw;
}

void Epwm_Fw_SetSecPulseFinal(uint16_t sec_pw)
{
    epwm_fw_state.final_sec_pulse_ns = sec_pw;
}

void Epwm_Fw_Set_Pri_Sec_PhaseFinal(int16_t pri_sec_phase)
{
    epwm_fw_state.final_phase_pri_sec_ns = pri_sec_phase;
}

//will update all final phase shift variables
void Epwm_Fw_Set_SetPhaseShiftsFinal(uint16_t pri_pw, int16_t pri_sec_phase, uint16_t sec_pw)
{
    epwm_fw_state.final_prim_pulse_ns = pri_pw;
    epwm_fw_state.final_sec_pulse_ns = sec_pw;
    epwm_fw_state.final_phase_pri_sec_ns = pri_sec_phase;
}

//to update all phase values to current status variables in struct
void Epwm_Fw_UpdatePhase(void)
{
    int8_t pri_step = 0, sec_step = 0, phase = 0;
    uint8_t updateFlag = 0;
    int16_t priPulse = epwm_fw_state.curr_prim_pulse_ns;
    int16_t secPulse = epwm_fw_state.curr_sec_pulse_ns;
    int16_t pri_sec_phase = epwm_fw_state.curr_phase_pri_sec_ns;


    if(epwm_fw_state.final_prim_pulse_ns != priPulse)
    {
        updateFlag = 1;
        //will check when final value is more than next step of current value
        if(epwm_fw_state.final_prim_pulse_ns > priPulse + EPWM_PRIM_PW_STEP)
        {
            pri_step += EPWM_PRIM_PW_STEP;
        }
        //will check when final value is less than next step of current value
        else if(epwm_fw_state.final_prim_pulse_ns < priPulse - EPWM_PRIM_PW_STEP)
        {
            pri_step -= EPWM_PRIM_PW_STEP;
        }
        //when step value is larger than difference between final and current
        else
        {
            pri_step = epwm_fw_state.final_prim_pulse_ns - priPulse;
        }

        if (priPulse + pri_step >= 0 && priPulse <= 5000)
        {
//            Epwm_Fw_SetPulseWidthPri(priPulse + pri_step);
            priPulse += pri_step;
        }
        else
        {
            /* error */
        }

    }


    if(epwm_fw_state.final_sec_pulse_ns != epwm_fw_state.curr_sec_pulse_ns)
    {
        updateFlag = 1;
        //will check when final value is more than next step of current value
        if(epwm_fw_state.final_sec_pulse_ns > secPulse + EPWM_SEC_PW_STEP)
        {
            sec_step += EPWM_SEC_PW_STEP;
        }
        //will check when final value is less than next step of current value
        else if(epwm_fw_state.final_sec_pulse_ns < secPulse - EPWM_SEC_PW_STEP)
        {
            sec_step -= EPWM_SEC_PW_STEP;
        }
        //when the step value is larger than difference between final and current
        else
        {
            sec_step = epwm_fw_state.final_sec_pulse_ns - secPulse;
        }


        if (secPulse + sec_step >= 0 && secPulse <= 5000)
        {
//            Epwm_Fw_SetPulseWidthSec(secPulse + sec_step);
            secPulse += sec_step;
        }
        else
        {
            /* error */
        }
    }

    if(epwm_fw_state.final_phase_pri_sec_ns != pri_sec_phase)
    {
        updateFlag = 1;
        //will check when final value is more than next step of current value
        if(epwm_fw_state.final_phase_pri_sec_ns > pri_sec_phase + EPWM_PHASE_SHIFT_STEP)
        {
            phase += EPWM_PHASE_SHIFT_STEP;
        }
        //will check when final value is less than next step of current value
        else if(epwm_fw_state.final_phase_pri_sec_ns < pri_sec_phase - EPWM_PHASE_SHIFT_STEP)
        {
            phase -= EPWM_PHASE_SHIFT_STEP;
        }
        //when the step change is larger than the difference between final and current
        else
        {
            phase = epwm_fw_state.final_phase_pri_sec_ns - pri_sec_phase;
        }


        if (pri_sec_phase + phase >= 0 && pri_sec_phase <= 2500)
        {
//            Epwm_Fw_SetPhaseShift_Pri_Sec(pri_sec_phase + phase);
            pri_sec_phase += phase;
        }
        else
        {
            /* error */
        }
    }

    if(updateFlag == TRUE)
    {
        Epwm_Fw_SetPhaseShifts(priPulse, pri_sec_phase, secPulse);
    }

}
